@@ -14,7 +14,7 @@ class StateMachine():
     sm.arrive("[Init]") # Init状態は作っておいてください
     """
 
-    def __init__(self, context=None, state_creator_dict={}, transition_dict={}):
+    def __init__(self, context=None, state_creator_dict={}, transition_dict={}, intermachine=None, name=None):
         """初期化
 
         Parameters
@@ -31,9 +31,12 @@ class StateMachine():
         self._state_creator_dict = state_creator_dict
         self._transition_dict = transition_dict
         self._verbose = False
-        self._state = None
         self._edge_path = []
-        self._lines_getter = None
+        self._lines_getter = None  # 標準入力とか１個しかないけど
+        self._state = None
+        self._is_terminate = False  # 永遠に停止
+        self._intermachine = intermachine
+        self._name = name
 
     @property
     def context(self):
@@ -74,8 +77,19 @@ class StateMachine():
     def verbose(self, val):
         self._verbose = val
 
-    def on_line(self, line):
-        pass
+    @property
+    def is_terminate(self):
+        """永遠に停止"""
+        return self._is_terminate
+
+    @is_terminate.setter
+    def is_terminate(self, val):
+        self._is_terminate = val
+
+    @property
+    def name(self):
+        """他のステートマシンと区別するためのキーとして使われます"""
+        return self._name
 
     def start(self, next_state_name):
         """まず state_machine.arrive(...) を行い、
@@ -83,7 +97,11 @@ class StateMachine():
         leave(...) に渡す line 引数は arrive(...) から返しますが、
         代わりに None を返すと self.lines_getter() が実行されます。
         self.lines_getter() は、 line のリストを返す関数です。
-        self.lines_getter() が None を返すとループを抜けます"""
+        self.lines_getter() が None を返すとループを抜けます
+        """
+        if self._is_terminate:
+            return
+
         self.arrive_sequence(next_state_name)
         self.leave_and_loop()
 
@@ -92,7 +110,11 @@ class StateMachine():
         そのあと arrive(...), leave(...) のペアを無限に繰り返します。
         leave(...) に渡す line 引数は、
         lines_getter() を実行することでリストで取得できるようにしてください。
-        lines_getter() が None を返すとループを抜けます"""
+        lines_getter() が None を返すとループを抜けます
+        """
+        if self._is_terminate:
+            return
+
         while True:
             lines = self.lines_getter()
             if lines is None:
@@ -115,6 +137,9 @@ class StateMachine():
         str : next_state_name
             次の状態の名前
         """
+        if self._is_terminate:
+            return
+
         interrupt_line = self.arrive(next_state_name)
 
         # interrupt_line の指定があったら、次の leave をすぐ行います
@@ -123,6 +148,12 @@ class StateMachine():
 
             interrupt_line = self.arrive(next_state_name)
 
+    def _alternate_state_machine_name(self):
+        if self.name is None:
+            return "[[state_machine]]"
+        else:
+            return self.name
+
     def arrive(self, next_state_name):
         """指定の状態に遷移します
         entryコールバック関数を呼び出します。
@@ -138,19 +169,26 @@ class StateMachine():
             ただちに leave に渡したい引数。無ければ None
         """
 
+        if self._is_terminate:
+            return
+
         if self.verbose:
             edge_path = '.'.join(self._edge_path)
-            print(f"[state_machine] Arrive to {next_state_name} {edge_path}")
+            print(
+                f"{self._alternate_state_machine_name()} Arrive to {next_state_name} {edge_path}")
 
         if next_state_name in self._state_creator_dict:
             # 次のステートへ引継ぎ
             self._state = self._state_creator_dict[next_state_name]()
 
-            req = Request(self._context, self._edge_path, None)
+            req = Request(context=self._context,
+                          edge_path=self._edge_path,
+                          line=None,
+                          intermachine=self._intermachine)
             interrupt_line = self._state.entry(req)
             if interrupt_line and self.verbose:
                 print(
-                    f"[state_machine] Arrive interrupt_line={interrupt_line}")
+                    f"{self._alternate_state_machine_name()} Arrive interrupt_line={interrupt_line}")
 
             return interrupt_line
 
@@ -173,11 +211,18 @@ class StateMachine():
         str
             次の状態の名前
         """
+        if self._is_terminate:
+            return
 
         if self.verbose:
-            print(f"[state_machine] Leave line={line}")
+            print(
+                f"{self._alternate_state_machine_name()} Leave line={line}")
 
-        req = Request(self._context, self.edge_path, line)
+        req = Request(
+            context=self._context,
+            edge_path=self.edge_path,
+            line=line,
+            intermachine=self._intermachine)
         next_edge_name = self._state.exit(req)
 
         # 例えば [Apple]ステート に居るとき ----Banana----> エッジに去るということは、
@@ -192,12 +237,21 @@ class StateMachine():
         #     }
         # }
         #
+        # "[Apple]": {
+        #     "----Banana---->" : None
+        # }
+        #
         # といった方法で値を取ってきます。
-        # 値は "[Zebra]"文字列かも知れませんし、 "----Cherry---->"ディクショナリーかもしれません。
+        # 値は "[Zebra]"文字列かも知れませんし、 "----Cherry---->"ディクショナリーかもしれませんし、
+        # None かもしれません。
 
         # まずはカレントステートを指定してディクショナリーを取ってきましょう
         if self.state.name in self._transition_dict:
             curr_dict = self._transition_dict[self.state.name]
+            if curr_dict is None:
+                self._is_terminate = True
+                self.on_terminate(req)
+                return
         else:
             raise ValueError(
                 f"Current state is not found. name=[{self.state.name}]")
@@ -206,6 +260,10 @@ class StateMachine():
         for i, edge in enumerate(self._edge_path):
             if edge in curr_dict:
                 curr_dict = curr_dict[edge]
+                if curr_dict is None:
+                    self._is_terminate = True
+                    self.on_terminate(req)
+                    return
             else:
                 raise ValueError(
                     f"Edge[{i}] is not found. name=[{edge}] path=[{self._edge_path}]")
@@ -228,3 +286,16 @@ class StateMachine():
                 f"Next edge is not found. name=[{next_edge_name}] current state=[{self.state.name}] path=[{self._edge_path}]")
 
         return next_state_name
+
+    def on_line(self, line):
+        pass
+
+    def on_terminate(self, req):
+        """永遠に停止
+
+        Parameters
+        ----------
+        req : Request
+            ステートマシンからステートへ与えられる引数のまとまり
+        """
+        pass
